import './index.css';
import '@grapecity/wyn-report-viewer/dist/jsViewer.min.js'
import '@grapecity/wyn-report-viewer/dist/jsViewer.min.css'

function concatUrl(base, rest) {
    base = base.trim();
    rest = rest.trim();
    if (base.substr(base.length - 1) == '/') base = base.substr(0, base.length - 1);
    if (rest.substr(0, 1) == '/') rest = rest.substr(1);
    return `${base}/${rest}`
}

async function getReferenceToken(url, user, password) {
	const endpoint = concatUrl(url, 'connect/token')
	const resolveResponse = async (response) => {
        const jsonResponse = await response.json();
		if (jsonResponse.error) return null;
		return jsonResponse.access_token;
	}

	return await fetch(endpoint, {
		headers: {
			'Content-Type': 'application/x-www-form-urlencoded',
			Accept: '*/*',
		},
		method: 'post',
		body: `grant_type=password&username=${user}&password=${password}&client_id=integration&client_secret=eunGKas3Pqd6FMwx9eUpdS7xmz`,
    }).then(async response => {
        let res = await resolveResponse(response)
        return res;
    }).catch(error => {
		alert(error);
		return null;
	});
}

async function getReportList(portalUrl, referenceToken) {
    const url = concatUrl(portalUrl, 'api/graphql')
    const init = {
        headers: {
            'Cache-Control': 'no-cache, no-store, must-revalidate',
            Accept: 'application/json',
            'content-type': 'application/json',
            'pragma': 'no-cache',
            'Reference-Token': referenceToken
        },
        method: 'post',
        body: JSON.stringify({ query: 'query { documenttypes(key:"rdl") { documents{ id, title } } }' })
    };
    const res = await fetch(url, init);
    if (!res.ok) return null
    let response = await res.json()
    let documents = response.data.documenttypes[0].documents;
    let list = documents.map(x => ({ name: x.title, id: x.id }))
    list.sort((x, y) => x.name < y.name ? -1 : 1 )
    return list
}

function createSignInForm() {
    const signInForm = {
        open: () => {
            document.getElementById('sign-in-root').hidden = false;
            document.getElementById("sign-in-username").focus();
        },
        close: () => { document.getElementById('sign-in-root').hidden = true },
        onSignIn: null
    }
    const signInButton = document.getElementById('sign-in-button');
    const signInAction = async () => {
        signInButton.disabled = true;
        let portalUrl = document.getElementById('sign-in-portal-url').value
        let username = document.getElementById('sign-in-username').value
        let password = document.getElementById('sign-in-password').value
        try {
            await signInForm.onSignIn(portalUrl, username, password);
            document.getElementById('sign-in-error').innerHTML = null
        }
        catch (err) {
            document.getElementById('sign-in-error').innerHTML = err.message
        }
        finally {
            signInButton.disabled = false;
        }
    };
    signInButton.onclick = signInAction
    document.getElementById('sign-in-root').addEventListener("keyup", (event) => {
        if (event.keyCode === 13)
            signInAction()
    });
    return signInForm
}

function createViewer(portalUrl, referenceToken, username) {
    document.getElementById('viewer-portal-url').innerHTML = portalUrl;
    document.getElementById('viewer-username').innerHTML = username;
    const reportsList = document.getElementById('wyn-report-list');
    const wynViewer = GrapeCity.ActiveReports.JSViewer.create({
        element: '#wyn-viewer-root',
        reportID: '',
        reportService: {
            url: portalUrl,
            securityToken: referenceToken,
        }
    });
    const viewer = {
        refreshReportsList: async () => { 
            const reports = await getReportList(portalUrl, referenceToken);
            reportsList.innerHTML = null;
            reports.forEach(report => {
                var item = document.createElement("li");
                item.value = report.id;
                var text = document.createElement("span")
                text.innerHTML = report.name;
                item.appendChild(text)
                item.className = 'wyn-report-list-item'
                reportsList.appendChild(item);
                item.addEventListener('click', () => {
                    let items = reportsList.children;
                    for (let i = 0; i < items.length; i++) {
                        items[i].classList.remove('active');
                    }
                    item.classList.add('active');
                    wynViewer.openReport(report.id)
                });
            });
        },
        open: () => { document.getElementById('viewer-root').hidden = false },
        close: () => { document.getElementById('viewer-root').hidden = true },
        destroy: () => { reportsList.innerHTML = null },
        onLogOut: null
    }
    document.getElementById('logout-button').onclick = () => { viewer.onLogOut() }
    return viewer;
}

function init() {
    const signInForm = createSignInForm();
    signInForm.onSignIn = async (portalUrl, username, password) => {
        const referenceToken = await getReferenceToken(portalUrl, username, password)
        if (!referenceToken)
            throw new Error('Invalid user name or password.')
        const viewer = createViewer(portalUrl, referenceToken, username);
        viewer.onLogOut = () => {
            signInForm.open();
            viewer.close();
            viewer.destroy();
        }
        viewer.refreshReportsList();
        signInForm.close();
        viewer.open();
    }
    signInForm.open();
}

init()